"""
This file is only here for backwards compatibility and there is a
good chance it will disppear soon. Please make use of:
:mod:`easymode.admin.models.fields` and :mod:`easymode.admin.forms.fields`
instead.
"""

from easymode.admin.forms.fields import *
from easymode.admin.models.fields import *