{% extends "reversion/object_history.html" %}
{% load i18n %}
{% load easyfilters %}
{% block content_title %}
    <h1>{{ title }}</h1>
{% endblock %}

{% block pretitle %}
    <small>{% trans '(items met status "draft")' %}</small>
{% endblock %}

{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href="../../../../">{% trans 'Home' %}</a> &rsaquo; 
    <a href="../../../">{{ app_label|capfirst }}</a> &rsaquo; 
    <a href="../../">{{ module_name }}</a> &rsaquo; 
    <a href="../">{{ object|truncatewords:"18" }}</a> &rsaquo; 
    {% trans 'Drafts' %}
</div>
{% endblock %}


{% block content %}
    {% block object-tools %}
                <ul class="object-tools">
                 {%if has_draft %}<li><a href=".." class="historylink">{% trans "Latest Draft" %}</a></li>{% endif %}
                 <li><a href="../current/" class="historylink">{% trans "Current" %}</a></li>
                 {% if draft_view %}
               <li><a href="../history/" class="historylink">{% trans "History" %}</a></li>
               {% else %}
               <li><a href="../drafts/" class="historylink">{% trans "Drafts" %}</a></li>
               {% endif %}
               {% if has_absolute_url %}<li><a href="../../../r/{{ content_type_id }}/{{ object_id }}/" class="viewsitelink">{% trans "View on site" %}</a></li>{% endif%}
               </ul>
    {% endblock %}

	<div id="content-main"> 

		<p>{% blocktrans %}Kies een draft uit de lijst om deze te kunnen publiceren.{% endblocktrans %}</p>
	
		<div class="module">
			{% if action_list %}
			    <table id="change-history">
			        <thead>
				        <tr>
				            <th scope="col">{% trans 'Date/time' %}</th>
				            <th scope="col">{% trans 'User' %}</th>
				            <th scope="col">{% trans 'Comment' %}</th>
				        </tr>
			        </thead>
			        <tbody>
				        {% for action in action_list %}
					        <tr>
					            <th scope="row"><a href="{{action.url|strip_locale}}">{{action.revision.date_created|date:_("DATETIME_FORMAT")}}</a></th>
					            <td>{{action.revision.user.username}}{% if action.revision.user.first_name %} ({{action.revision.user.first_name}} {{action.revision.user.last_name}}){% endif %}</td>
					            <td>{{action.revision.comment|default:""}}</td>
					        </tr>
				        {% endfor %}
			        </tbody>
			    </table>
			{% else %}
			    <p>{% trans "This object doesn't have a change history. It probably wasn't added via this admin site." %}</p>
			{% endif %}
		</div>
	</div>
{% endblock %}
